************************************************************************
* File (process_cc_1loop_decay.F).
************************************************************************
* This is the FORTRAN subroutine module to calculate 1-loop EW and QCD
* radiative correction for the Charged Current Drell-Yan processes
* (decay).
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2010.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine Process_Decay
     &(born,born_sd,virt,virt_sd,viso,viso_sd,brdq,brdq_sd,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 ihard,ome,pid
      integer*4 ncall1,itmx1,itmx2
      integer*4 nrdm,nc1(10),it1(10),it2(10)
      real*8 born_sd,virt_sd,viso,viso_sd,brdq,brdq_sd,hard_sd
      real*8 virt_ew,virt_ew_sd,viso_ew,viso_qcd,brdq_ew,brdq_qcd,
     &     hard_ew,hard_qcd
      real*8 viso_ew_sd,viso_qcd_sd,brdq_ew_sd,brdq_qcd_sd,hard_ew_sd,
     &     hard_qcd_sd

      common /ihard/ ihard
      common /pid/ pid
      common /vegas_param/ nrdm,nc1,it1,it2

      born        = 0d0
      born_sd     = 0d0
      virt        = 0d0
      virt_sd     = 0d0
      virt_ew     = 0d0
      virt_ew_sd  = 0d0
      viso_ew     = 0d0
      viso_ew_sd  = 0d0
      viso_qcd    = 0d0
      viso_qcd_sd = 0d0
      brdq        = 0d0
      brdq_sd     = 0d0
      brdq_ew     = 0d0
      brdq_ew_sd  = 0d0
      brdq_qcd    = 0d0
      brdq_qcd_sd = 0d0
      hard        = 0d0
      hard_sd     = 0d0
      hard_ew     = 0d0
      hard_ew_sd  = 0d0
      hard_qcd    = 0d0
      hard_qcd_sd = 0d0

*     born
      iborn = 1
      ncall1 = nc1(1)
      itmx1  = it1(1)
      itmx2  = it2(1)
      call ProcessBorn_D
     &     (nrdm,ncall1,itmx1,itmx2,born,born_sd)

      print *, "[Born]         = ", born, born_sd
      if (iqed.eq.0.and.iew.eq.0.and.iqcd.eq.0) stop

      print *, ""
      print *, "omega(GeV)     = ", omega

*     EW part
      if (iew.ne.0) then
*     virt (Weak)
         iborn = 0
         ncall1 = nc1(2) 
         itmx1  = it1(2)
         itmx2  = it2(2)
         call ProcessVirt_D_EW
     &        (nrdm,ncall1,itmx1,itmx2,virt_ew,virt_ew_sd)
      endif

      if (iqed.ne.0) then
*     virt+soft (QED)
         iborn = 0
         ncall1 = nc1(3)
         itmx1  = it1(3)
         itmx2  = it2(3)
         call ProcessViSo_D_EW
     &        (nrdm,ncall1,itmx1,itmx2,viso_ew,viso_ew_sd)

         if (ihard.eq.1) then
*     hard (QED), 2spr
            iborn = 0
            ncall1 = nc1(5)
            itmx1  = it1(5)
            itmx2  = it2(5)
            call ProcessHard_D_EW_2spr
     &           (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)

         elseif (ihard.eq.2) then
*     hard (QED), 5d
            iborn = 0
            ncall1 = nc1(6)
            itmx1  = it1(6)
            itmx2  = it2(6)
            call ProcessHard_D_EW
     &           (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)
         endif
      endif

*     QCD part
      if (iqcd.ne.0) then
*     virt+soft (QCD)
         iborn = 0
         ncall1 = nc1(7)
         itmx1  = it1(7)
         itmx2  = it2(7)
         call ProcessViSo_D_QCD
     &        (nrdm,ncall1,itmx1,itmx2,viso_qcd,viso_qcd_sd)

         if (ihard.eq.1) then
*     hard (QCD), 2spr
            iborn = 0
            ncall1 = nc1(9)
            itmx1  = it1(9)
            itmx2  = it2(9)
            call ProcessHard_D_QCD_2spr
     &           (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)
            
         elseif (ihard.eq.2) then
*     hard, 5d
            iborn = 0
            ncall1 = nc1(10)
            itmx1  = it1(10)
            itmx2  = it2(10)
            call ProcessHard_D_QCD
     &           (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)
         endif
      endif

      virt    = virt_ew
      virt_sd = virt_ew_sd
      viso    = viso_ew + viso_qcd
      viso_sd = dsqrt(viso_ew_sd**2 + viso_qcd_sd**2)
      brdq    = brdq_ew + brdq_qcd
      brdq_sd = dsqrt(brdq_ew_sd**2 + brdq_qcd_sd**2)
      hard    = hard_ew + hard_qcd
      hard_sd = dsqrt(hard_ew_sd**2 + hard_qcd_sd**2)

      return
      end
************************************************************************
* Born, decay, EW
************************************************************************
      subroutine ProcessBorn_D
     &     (nrdm,ncall1,itmx1,itmx2,born,born_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 born_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_born_d_ew_2c

      xl(1) = (mf+mf1)**2
      xu(1) = (mtp)**2
      xl(2) = -1d0
      xu(2) = +1d0

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_born_d_ew_2c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_born_d_ew_2c, avgi, sd, chi2a)
      born = avgi
      born_sd = sd

      return
      end
************************************************************************
      real*8 function function_born_d_ew_2c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      s     = x(1)
      cosf  = x(2)

      t = 1d0/2*(mtp**2-s)*(1-cosf)
      u = 1d0/2*(mtp**2-s)*(1+cosf)

      function_born_d_ew_2c = 0d0
      if     (pid.eq.19) then
         call cc_br_2122_1112 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.20) then
         call cc_br_2221_1211 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.21) then
         call cc_br_2122_1516 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.22) then
         call cc_br_2221_1615 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.23) then
         call cc_br_2122_1314 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.24) then
         call cc_br_2221_1413 (-s,-t,-u,born,soft,hard)
      endif
      function_born_d_ew_2c = born

      return
      end
************************************************************************
* Virt (Weak), decay, EW
************************************************************************
      subroutine ProcessVirt_D_EW(nrdm,ncall1,itmx1,itmx2,virt,virt_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 virt_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_virt_d_ew_2c

      x1cut = 1d-10
      xl(1) = (mf+mf1)**2
      xu(1) = (mtp)**2
      xl(2) = -1d0+x1cut
      xu(2) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_virt_d_ew_2c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_virt_d_ew_2c, avgi, sd, chi2a)
      virt = avgi
      virt_sd = sd

      return
      end
************************************************************************
      real*8 function function_virt_d_ew_2c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      nc = 1d0
      s     = x(1)
      cosf  = x(2)

      t = 1d0/2*(mtp**2-s)*(1-cosf)
      u = 1d0/2*(mtp**2-s)*(1+cosf)

      function_virt_d_ew_2c = 0d0
      width = 0d0
      if     (pid.eq.19) then
         call cc_br_2122_1112 (-s,-t,-u,born,soft,hard)
         call cc_tw_2122_1112 (s,t,u,width)
      elseif (pid.eq.20) then
         call cc_br_2221_1211 (-s,-t,-u,born,soft,hard)
         call cc_tw_2221_1211 (s,t,u,width)
      elseif (pid.eq.21) then
         call cc_br_2122_1516 (-s,-t,-u,born,soft,hard)
         call cc_tw_2122_1516 (s,t,u,width)
      elseif (pid.eq.22) then
         call cc_br_2221_1615 (-s,-t,-u,born,soft,hard)
         call cc_tw_2221_1615 (s,t,u,width)
      elseif (pid.eq.23) then
         call cc_br_2122_1314 (-s,-t,-u,born,soft,hard)
         call cc_tw_2122_1314 (s,t,u,width)
      elseif (pid.eq.24) then
         call cc_br_2221_1413 (-s,-t,-u,born,soft,hard)
         call cc_tw_2221_1413 (s,t,u,width)
      endif
      function_virt_d_ew_2c = width - born

      return
      end
************************************************************************
* Virt+Soft (QED), decay, EW
************************************************************************
      subroutine ProcessViSo_D_EW(nrdm,ncall1,itmx1,itmx2,viso,viso_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 viso,viso_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_viso_d_ew_2c

      xl(1) = (mf+mf1)**2
      xu(1) = (mtp)**2
      xl(2) = -1d0
      xu(2) = +1d0

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_viso_d_ew_2c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_viso_d_ew_2c, avgi, sd, chi2a)
      viso = avgi
      viso_sd = sd

      return
      end
************************************************************************
      real*8 function function_viso_d_ew_2c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      s     = x(1)
      cosf  = x(2)

      t = 1d0/2*(mtp**2-s)*(1-cosf)
      u = 1d0/2*(mtp**2-s)*(1+cosf)

      function_viso_d_ew_2c = 0d0
      if     (pid.eq.19) then
         call cc_br_2122_1112 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.20) then
         call cc_br_2221_1211 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.21) then
         call cc_br_2122_1516 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.22) then
         call cc_br_2221_1615 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.23) then
         call cc_br_2122_1314 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.24) then
         call cc_br_2221_1413 (-s,-t,-u,born,soft,hard)
      endif
      function_viso_d_ew_2c = soft

      return
      end
************************************************************************
* Hard (2spr), decay, EW
************************************************************************
      subroutine ProcessHard_D_EW_2spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_hard_d_ew_2spr

      xl(1) = (mf+mf1)**2            ! s
      xu(1) = (mtp)**2
      xl(2) = (mf+mf1)**2            ! spr
      xu(2) = (mtp)**2
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_hard_d_ew_2spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_hard_d_ew_2spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_d_ew_2spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      s   = x(1)
      spr = x(2)

      function_hard_d_ew_2spr = 0d0
      if (spr.ge.s-2d0*sqrt(s)*omega) return
      if     (pid.eq.19) then
         call cc_ha_2122_1112_1spr (s,spr,hard)
      elseif (pid.eq.20) then
         call cc_ha_2221_1211_1spr (s,spr,hard)
      elseif (pid.eq.21) then
         call cc_ha_2122_1516_1spr (s,spr,hard)
      elseif (pid.eq.22) then
         call cc_ha_2221_1615_1spr (s,spr,hard)
      elseif (pid.eq.23) then
         call cc_ha_2122_1314_1spr (s,spr,hard)
      elseif (pid.eq.24) then
         call cc_ha_2221_1413_1spr (s,spr,hard)
      endif
      function_hard_d_ew_2spr = hard
      
      return
      end
************************************************************************
* Hard (5d), decay, EW
************************************************************************
      subroutine ProcessHard_D_EW(nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_hard_d_ew_5d

      xl(1) = (mf+mf1)**2            ! s
      xu(1) = (mtp-mbt)**2
      xl(2) = (mf+mf1)**2            ! spr
      xu(2) = (mtp-mbt)**2
      xl(3) = 0d0
      xu(3) = 2d0*pi
      xl(4) = -1d0
      xu(4) = +1d0
      xl(5) = -1d0
      xu(5) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (5, function_hard_d_ew_5d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (5, function_hard_d_ew_5d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_d_ew_5d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid

      s    = x(1)
      spr  = x(2)
      ph3  = x(3)
      cth3 = x(4)
      cth5 = x(5)

      function_hard_d_ew_5d = 0d0
      if (spr.ge.s-2d0*sqrt(s)*omega) return
      if     (pid.eq.19) then
         call cc_ha_2122_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.20) then
         call cc_ha_2221_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.21) then
         call cc_ha_2122_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.22) then
         call cc_ha_2221_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.23) then
         call cc_ha_2122_1314 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.24) then
         call cc_ha_2221_1413 (s,spr,ph3,cth3,cth5,hard)
      endif
      function_hard_d_ew_5d = hard

      return
      end
************************************************************************
* Virt+Soft, decay, QCD
************************************************************************
      subroutine ProcessViSo_D_QCD(nrdm,ncall1,itmx1,itmx2,viso,viso_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 viso,viso_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_viso_d_qcd_2c

      x1cut = 1d-10
      xl(1) = (mf+mf1)**2
      xu(1) = (mtp)**2
      xl(2) = -1d0+x1cut
      xu(2) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_viso_d_qcd_2c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_viso_d_qcd_2c, avgi, sd, chi2a)
      viso = avgi
      viso_sd = sd

      return
      end
************************************************************************
      real*8 function function_viso_d_qcd_2c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      s     = x(1)
      cosf  = x(2)

      t = 1d0/2*(mtp**2-s)*(1-cosf)
      u = 1d0/2*(mtp**2-s)*(1+cosf)

      function_viso_d_qcd_2c = 0d0
      if     (pid.eq.19) then
         call cc_qcd_br_2122_1112 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.20) then
         call cc_qcd_br_2221_1211 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.21) then
         call cc_qcd_br_2122_1516 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.22) then
         call cc_qcd_br_2221_1615 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.23) then
         call cc_qcd_br_2122_1314 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.24) then
         call cc_qcd_br_2221_1413 (-s,-t,-u,born,virt,hard)
      endif
      function_viso_d_qcd_2c = virt

      return
      end
************************************************************************
* Hard (2spr), decay, QCD
************************************************************************
      subroutine ProcessHard_D_QCD_2spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_hard_d_qcd_2spr

      xl(1) = (mf+mf1)**2            ! s
      xu(1) = (mtp)**2
      xl(2) = (mf+mf1)**2            ! spr
      xu(2) = (mtp)**2

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (2, function_hard_d_qcd_2spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (2, function_hard_d_qcd_2spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_d_qcd_2spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /pid/ pid

      s   = x(1)
      spr = x(2)

      function_hard_d_qcd_2spr = 0d0
      if (spr.ge.s-2d0*sqrt(s)*omega) return
      if     (pid.eq.19) then
         call cc_qcd_ha_2122_1112_1spr (s,spr,hard)
      elseif (pid.eq.20) then
         call cc_qcd_ha_2221_1211_1spr (s,spr,hard)
      elseif (pid.eq.21) then
         call cc_qcd_ha_2122_1516_1spr (s,spr,hard)
      elseif (pid.eq.22) then
         call cc_qcd_ha_2221_1615_1spr (s,spr,hard)
      elseif (pid.eq.23) then
         call cc_qcd_ha_2122_1314_1spr (s,spr,hard)
      elseif (pid.eq.24) then
         call cc_qcd_ha_2221_1413_1spr (s,spr,hard)
      endif
      function_hard_d_qcd_2spr = hard
      
      return
      end
************************************************************************
* Hard (5d), decay, QCD
************************************************************************
      subroutine ProcessHard_D_QCD(nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_hard_d_qcd_5d

      xl(1) = (mf+mf1)**2       ! s
      xu(1) = (mtp-mbt)**2
      xl(2) = (mf+mf1)**2            ! spr
      xu(2) = (mtp-mbt)**2
      xl(3) = 0d0
      xu(3) = 2d0*pi
      xl(4) = -1d0
      xu(4) = +1d0
      xl(5) = -1d0
      xu(5) = +1d0
     
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (5, function_hard_d_qcd_5d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (5, function_hard_d_qcd_5d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_d_qcd_5d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid

      s    = x(1)
      spr  = x(2)
      ph3  = x(3)
      cth3 = x(4)
      cth5 = x(5)

      function_hard_d_qcd_5d = 0d0
      if (spr.ge.s-2d0*sqrt(s)*omega) return
      if     (pid.eq.19) then
         call cc_qcd_ha_2122_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.20) then
         call cc_qcd_ha_2221_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.21) then
         call cc_qcd_ha_2122_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.22) then
         call cc_qcd_ha_2221_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.23) then
         call cc_qcd_ha_2122_1314 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.24) then
         call cc_qcd_ha_2221_1413 (s,spr,ph3,cth3,cth5,hard)
      endif
      function_hard_d_qcd_5d = hard

      return
      end
************************************************************************
